"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useZodios = void 0;
const react_1 = require("react");
const react_query_1 = require("react-query");
const provider_1 = require("./provider");
const utils_1 = require("../utils");
/**
 * React hook to call an api "get" endpoint.
 * It's a thin wrapper for react-query and useContext.
 * It should not be used without wrapping it in another hook @see example
 * else you will not have nice autocompletions in your IDE.
 * It should be used in combination with the ZodiosProvider
 * @param apiName - the name of the api used in Zodios Provider
 * @param path - the path to api endpoint
 * @param config - the config to setup axios options and parameters
 * @param queryOptions - the query options to setup react-query options
 * @returns the response validated with zod
 *
 * @example ```typescript
 *   type Api = ApiOf<typeof myZodiosClient>;
 *   function useJsonPlaceholder<Path extends Paths<Api, "get">>(
 *     path: Path,
 *     config?: ZodiosRequestOptions<Api, "get", Path>
 *   ) {
 *     return useZodios("https://jsonplaceholder.typicode.com", path, config);
 *   }
 * ```
 */
function useZodios(apiName, path, config, queryOptions) {
    const api = (0, react_1.useContext)(provider_1.ZodiosContext)[apiName];
    if (!api)
        throw new Error(`can't find api ${apiName}`);
    const zodios = api;
    const params = (0, utils_1.pick)(config, [
        "params",
        "queries",
    ]);
    const keys = [apiName, path, params];
    const query = () => zodios.get(path, config);
    return (0, react_query_1.useQuery)(keys, query, queryOptions);
}
exports.useZodios = useZodios;
